@@ -227,6 +227,7 @@ typedef struct _LogProtoFileWriter
   gint buf_count;
   gint fd;
   gint sum_len;
+  gboolean fsync;
   struct iovec buffer[0];
 } LogProtoFileWriter;
 
@@ -254,6 +255,8 @@ log_proto_file_writer_flush(LogProto *s)
 
   lseek(self->fd, 0, SEEK_END);
   rc = writev(self->fd, self->buffer, self->buf_count);
+  if (rc > 0 && self->fsync)
+    fsync(self->fd);
 
   if (rc < 0)
     {
@@ -340,6 +343,8 @@ log_proto_file_writer_post(LogProto *s, guchar *msg, gsize msg_len, gboolean *co
       gint len = self->partial_len - self->partial_pos;
 
       rc = write(self->fd, self->partial + self->partial_pos, len);
+      if (rc > 0 && self->fsync)
+        fsync(self->fd);
       if (rc < 0)
         {
           goto write_error;
@@ -401,7 +406,7 @@ log_proto_file_writer_prepare(LogProto *s, gint *fd, GIOCondition *cond)
 }
 
 LogProto *
-log_proto_file_writer_new(LogTransport *transport, gint flush_lines)
+log_proto_file_writer_new(LogTransport *transport, gint flush_lines, gboolean fsync)
 {
   if (flush_lines == 0)
     /* the flush-lines option has not been specified, use a default value */
@@ -417,6 +422,7 @@ log_proto_file_writer_new(LogTransport *transport, gint flush_lines)
 
   self->fd = transport->fd;
   self->buf_size = flush_lines;
+  self->fsync = fsync;
   self->super.prepare = log_proto_file_writer_prepare;
   self->super.post = log_proto_file_writer_post;
   self->super.flush = log_proto_file_writer_flush;
@@ -476,9 +482,64 @@ struct _LogProtoBufferedServer
   gint init_buffer_size;
   guchar *buffer;
   GSockAddr *prev_saddr;
-  LogProtoStatus status;
 };
 
+
+/*
+ * log_proto_get_fixed_encoding_scale:
+ *
+ * This function returns the number of bytes of a single character in the
+ * encoding specified by the @encoding parameter, provided it is listed in
+ * the limited set hard-wired in the fixed_encodings array above.
+ *
+ * syslog-ng sometimes needs to calculate the size of the original, raw data
+ * that relates to its already utf8 converted input buffer.  For that the
+ * slow solution is to actually perform the utf8 -> raw conversion, however
+ * since we don't really need the actual conversion, just the size of the
+ * data in bytes we can be faster than that by multiplying the number of
+ * input characters with the size of the character in the known
+ * fixed-length-encodings in the list above.
+ *
+ * This function returns 0 if the encoding is not known, in which case the
+ * slow path is to be executed.
+ */
+gint
+log_proto_get_char_size_for_fixed_encoding(const gchar *encoding)
+{
+  static struct
+  {
+    const gchar *prefix;
+    gint scale;
+  } fixed_encodings[] = {
+    { "ascii", 1 },
+    { "us-ascii", 1 },
+    { "iso-8859", 1 },
+    { "iso8859", 1 },
+    { "latin", 1 },
+    { "ucs2", 2 },
+    { "ucs-2", 2 },
+    { "ucs4", 4 },
+    { "ucs-4", 4 },
+    { "koi", 1 },
+    { "unicode", 2 },
+    { "windows", 1 },
+    { "wchar_t", sizeof(wchar_t) },
+    { NULL, 0 }
+  };
+  gint scale = 0;
+  gint i;
+
+  for (i = 0; fixed_encodings[i].prefix; i++)
+   {
+     if (strncasecmp(encoding, fixed_encodings[i].prefix, strlen(fixed_encodings[i].prefix)) == 0)
+       {
+         scale = fixed_encodings[i].scale;
+         break;
+       }
+   }
+  return scale;
+}
+
 static LogProtoBufferedServerState *
 log_proto_buffered_server_get_state(LogProtoBufferedServer *self)
 {
@@ -523,35 +584,46 @@ log_proto_buffered_server_convert_from_raw(LogProtoBufferedServer *self, const g
           switch (errno)
             {
             case EINVAL:
-              /* Incomplete text, do not report an error, rather try to read again */
-              state->pending_buffer_end = state->buffer_size - avail_out;
-
-              if (avail_in > 0)
+              if ((self->super.flags & LPBS_RECORD) == 0)
                 {
-                  if (avail_in > sizeof(state->raw_buffer_leftover))
+                  /* Incomplete text, do not report an error, rather try to read again */
+                  state->pending_buffer_end = state->buffer_size - avail_out;
+
+                  if (avail_in > 0)
                     {
-                      msg_error("Invalid byte sequence, the remaining raw buffer is larger than the supported leftover size",
+                      if (avail_in > sizeof(state->raw_buffer_leftover))
+                        {
+                          msg_error("Invalid byte sequence, the remaining raw buffer is larger than the supported leftover size",
+                                    evt_tag_str("encoding", self->super.encoding),
+                                    evt_tag_int("avail_in", avail_in),
+                                    evt_tag_int("leftover_size", sizeof(state->raw_buffer_leftover)),
+                                    NULL);
+                          goto error;
+                        }
+                      memcpy(state->raw_buffer_leftover, raw_buffer, avail_in);
+                      state->raw_buffer_leftover_size = avail_in;
+                      state->raw_buffer_size -= avail_in;
+                      msg_trace("Leftover characters remained after conversion, delaying message until another chunk arrives",
                                 evt_tag_str("encoding", self->super.encoding),
                                 evt_tag_int("avail_in", avail_in),
-                                evt_tag_int("leftover_size", sizeof(state->raw_buffer_leftover)),
                                 NULL);
-                      goto error;
+                      goto success;
                     }
-                  memcpy(state->raw_buffer_leftover, raw_buffer, avail_in);
-                  state->raw_buffer_leftover_size = avail_in;
-                  state->raw_buffer_size -= avail_in;
-                  msg_trace("Leftover characters remained after conversion, delaying message until another chunk arrives",
+                }
+              else
+                {
+                  msg_error("Byte sequence too short, cannot convert an individual frame in its entirety",
                             evt_tag_str("encoding", self->super.encoding),
                             evt_tag_int("avail_in", avail_in),
                             NULL);
-                  goto success;
+                  goto error;
                 }
               break;
             case E2BIG:
               state->pending_buffer_end = state->buffer_size - avail_out;
               /* extend the buffer */
 
-              if ((state->buffer_size < self->max_buffer_size))
+              if (state->buffer_size < self->max_buffer_size)
                 {
                   state->buffer_size *= 2;
                   if (state->buffer_size > self->max_buffer_size)
@@ -663,12 +735,13 @@ log_proto_buffered_server_apply_state(LogProtoBufferedServer *self, PersistEntry
         }
       else
         {
-          if (state->raw_buffer_size > self->init_buffer_size)
+          if (state->raw_buffer_size > self->max_buffer_size)
             {
-              msg_notice("Invalid LogProtoBufferedServerState.raw_buffer_size, larger than init_buffer_size, restarting from the beginning",
+              msg_notice("Invalid LogProtoBufferedServerState.raw_buffer_size, larger than max_buffer_size, restarting from the beginning",
                          evt_tag_str("state", persist_name),
                          evt_tag_int("raw_buffer_size", state->raw_buffer_size),
                          evt_tag_int("init_buffer_size", self->init_buffer_size),
+                         evt_tag_int("max_buffer_size", self->max_buffer_size),
                          NULL);
               goto error;
             }
@@ -1042,7 +1115,7 @@ log_proto_buffered_server_fetch(LogProto *s, const guchar **msg, gsize *msg_len,
   gint rc;
   guchar *raw_buffer = NULL;
   LogProtoBufferedServerState *state = log_proto_buffered_server_get_state(self);
-  LogProtoStatus result = self->status;
+  LogProtoStatus result = self->super.status;
 
   if (G_UNLIKELY(!self->buffer))
     {
@@ -1053,11 +1126,6 @@ log_proto_buffered_server_fetch(LogProto *s, const guchar **msg, gsize *msg_len,
   if (sa)
     *sa = NULL;
 
-  if (self->status != LPS_SUCCESS)
-    {
-      goto exit;
-    }
-
   if (log_proto_buffered_server_fetch_from_buf(self, msg, msg_len, FALSE))
     {
       if (sa && self->prev_saddr)
@@ -1118,14 +1186,14 @@ log_proto_buffered_server_fetch(LogProto *s, const guchar **msg, gsize *msg_len,
 
               /* we set self->status explicitly as we want to return
                * LPS_ERROR on the _next_ invocation, not now */
-              self->status = LPS_ERROR;
+              self->super.status = LPS_ERROR;
               if (log_proto_buffered_server_fetch_from_buf(self, msg, msg_len, TRUE))
                 {
                   if (sa && self->prev_saddr)
                     *sa = g_sockaddr_ref(self->prev_saddr);
                   goto exit;
                 }
-              result = self->status;
+              result = self->super.status;
               goto exit;
             }
         }
@@ -1144,14 +1212,14 @@ log_proto_buffered_server_fetch(LogProto *s, const guchar **msg, gsize *msg_len,
                   result = LPS_EOF;
                   goto exit;
                 }
-              self->status = LPS_EOF;
+              self->super.status = LPS_EOF;
               if (log_proto_buffered_server_fetch_from_buf(self, msg, msg_len, TRUE))
                 {
                   if (sa && self->prev_saddr)
                     *sa = g_sockaddr_ref(self->prev_saddr);
                   goto exit;
                 }
-              result = self->status;
+              result = self->super.status;
               goto exit;
             }
           else
@@ -1193,7 +1261,7 @@ log_proto_buffered_server_fetch(LogProto *s, const guchar **msg, gsize *msg_len,
   /* result contains our result, but once an error happens, the error condition remains persistent */
   log_proto_buffered_server_put_state(self);
   if (result != LPS_SUCCESS)
-    self->status = result;
+    self->super.status = result;
   return result;
 }
 
@@ -1385,27 +1453,6 @@ find_eom(const guchar *s, gsize n)
   return NULL;
 }
 
-struct
-{
-  const gchar *prefix;
-  gint scale;
-} fixed_encodings[] = {
-  { "ascii", 1 },
-  { "us-ascii", 1 },
-  { "iso-8859", 1 },
-  { "iso8859", 1 },
-  { "latin", 1 },
-  { "ucs2", 2 },
-  { "ucs-2", 2 },
-  { "ucs4", 4 },
-  { "ucs-4", 4 },
-  { "koi", 1 },
-  { "unicode", 2 },
-  { "windows", 1 },
-  { "wchar_t", sizeof(wchar_t) },
-  { NULL, 0 }
-};
-
 /*
  * returns the number of bytes that represent the UTF8 encoding buffer
  * in the original encoding that the user specified.
@@ -1424,24 +1471,16 @@ log_proto_text_server_get_raw_size_of_buffer(LogProtoTextServer *self, const guc
 
   if (self->reverse_convert == ((GIConv) -1) && !self->convert_scale)
     {
-      gint i;
-
       /* try to speed up raw size calculation by recognizing the most
        * prominent character encodings and in the case the encoding
        * uses fixed size characters set that in self->convert_scale,
        * which in turn will speed up the reversal of the UTF8 buffer
        * size to raw buffer sizes.
        */
-      for (i = 0; fixed_encodings[i].prefix; i++)
-        {
-          if (strncasecmp(self->super.super.encoding, fixed_encodings[i].prefix, strlen(fixed_encodings[i].prefix) == 0))
-            {
-              self->convert_scale = fixed_encodings[i].scale;
-              break;
-            }
-        }
-      if (!fixed_encodings[i].prefix)
+      self->convert_scale = log_proto_get_char_size_for_fixed_encoding(self->super.super.encoding);
+      if (self->convert_scale == 0)
         {
+          /* this encoding is not known, do the conversion for real :( */
           self->reverse_convert = g_iconv_open(self->super.super.encoding, "utf-8");
         }
     }
@@ -1603,9 +1642,10 @@ log_proto_text_server_fetch_from_buf(LogProtoBufferedServer *s, const guchar *bu
   return result;
 }
 
-void
-log_proto_text_server_free(LogProtoTextServer *self)
+static void
+log_proto_text_server_free(LogProto *s)
 {
+  LogProtoTextServer *self = (LogProtoTextServer *) s;
   if (self->reverse_convert != (GIConv) -1)
     g_iconv_close(self->reverse_convert);
 
@@ -1619,6 +1659,7 @@ log_proto_text_server_init(LogProtoTextServer *self, LogTransport *transport, gi
   log_proto_buffered_server_init(&self->super, transport, max_msg_size * 6, max_msg_size, flags);
   self->super.fetch_from_buf = log_proto_text_server_fetch_from_buf;
   self->super.super.prepare = log_proto_text_server_prepare;
+  self->super.super.free_fn = log_proto_text_server_free;
   self->reverse_convert = (GIConv) -1;
 }
 
@@ -1688,7 +1729,7 @@ log_proto_record_server_new(LogTransport *transport, gint record_size, guint fla
 {
   LogProtoRecordServer *self = g_new0(LogProtoRecordServer, 1);
 
-  log_proto_buffered_server_init(&self->super, transport, record_size * 6, record_size, flags);
+  log_proto_buffered_server_init(&self->super, transport, record_size * 6, record_size, flags | LPBS_RECORD);
   self->super.fetch_from_buf = log_proto_record_server_fetch_from_buf;
   self->super.read_data = log_proto_record_server_read_data;
   self->record_size = record_size;
@@ -1722,7 +1763,7 @@ log_proto_dgram_server_new(LogTransport *transport, gint max_msg_size, guint fla
 {
   LogProtoRecordServer *self = g_new0(LogProtoRecordServer, 1);
 
-  log_proto_buffered_server_init(&self->super, transport, max_msg_size * 6, max_msg_size, flags | LPBS_IGNORE_EOF);
+  log_proto_buffered_server_init(&self->super, transport, max_msg_size * 6, max_msg_size, flags | LPBS_IGNORE_EOF | LPBS_RECORD);
   self->super.fetch_from_buf = log_proto_dgram_server_fetch_from_buf;
   return &self->super.super;
 }
@@ -1864,7 +1905,7 @@ log_proto_framed_server_fetch_data(LogProtoFramedServer *self, gboolean *may_rea
     {
       if (errno != EAGAIN)
         {
-          msg_error("Error reading frame header",
+          msg_error("Error reading RFC5428 style framed data",
                     evt_tag_int("fd", self->super.transport->fd),
                     evt_tag_errno("error", errno),
                     NULL);
