@@ -120,19 +120,20 @@ log_reader_work_finished(void *s)
 {
   LogReader *self = (LogReader *) s;
 
-  if (self->notify_code == 0)
+  if (self->notify_code)
     {
-      if (self->super.super.flags & PIF_INITIALIZED)
-        {
-          /* reenable polling the source assuming that we're still in
-           * business (e.g. the reader hasn't been uninitialized) */
+      gint notify_code = self->notify_code;
 
-          log_reader_start_watches(self);
-        }
+      self->notify_code = 0;
+      log_pipe_notify(self->control, &self->super.super, notify_code, self);
     }
-  else
+  if (self->super.super.flags & PIF_INITIALIZED)
     {
-      log_pipe_notify(self->control, &self->super.super, self->notify_code, self);
+      /* reenable polling the source assuming that we're still in
+       * business (e.g. the reader hasn't been uninitialized) */
+
+      log_proto_reset_error(self->proto);
+      log_reader_start_watches(self);
     }
   log_pipe_unref(&self->super.super);
 }
@@ -329,9 +330,6 @@ log_reader_start_watches(LogReader *self)
 
   log_proto_prepare(self->proto, &fd, &cond);
 
-  if (self->pollable_state < 0 && fd >= 0)
-    self->pollable_state = iv_fd_pollable(fd);
-
   if (self->options->follow_freq > 0)
     {
       /* follow freq specified (only the file source does that, go into timed polling */
@@ -345,18 +343,28 @@ log_reader_start_watches(LogReader *self)
                 NULL);
       return FALSE;
     }
-  else if (self->pollable_state > 0)
+  else
     {
       /* we have an FD, it is possible to poll it, register it  */
       self->fd_watch.fd = fd;
-      iv_fd_register(&self->fd_watch);
-    }
-  else
-    {
-      msg_error("Unable to determine how to monitor this fd, follow_freq() not set and it is not possible to poll it with the current ivykis polling method, try changing IV_EXCLUDE_POLL_METHOD environment variable",
-                evt_tag_int("fd", fd),
-                NULL);
-      return FALSE;
+      if (self->pollable_state < 0)
+        {
+          if (iv_fd_register_try(&self->fd_watch) == 0)
+            self->pollable_state = 1;
+          else
+            self->pollable_state = 0;
+        }
+      else if (self->pollable_state > 0)
+        {
+          iv_fd_register(&self->fd_watch);
+        }
+      else
+        {
+          msg_error("Unable to determine how to monitor this fd, follow_freq() not set and it is not possible to poll it with the current ivykis polling method, try changing IV_EXCLUDE_POLL_METHOD environment variable",
+                    evt_tag_int("fd", fd),
+                    NULL);
+          return FALSE;
+        }
     }
 
   log_reader_update_watches(self);
