@@ -539,9 +539,12 @@ afinet_dd_setup_socket(AFSocketDestDriver *s, gint fd)
 static gboolean
 afinet_dd_init(LogPipe *s)
 {
-  AFInetDestDriver *self G_GNUC_UNUSED = (AFInetDestDriver *) s;
+  AFInetDestDriver *self = (AFInetDestDriver *) s;
   gboolean success;
 
+  if (self->spoof_source)
+    self->super.flags &= ~AFSOCKET_KEEP_ALIVE;
+
   success = afsocket_dd_init(s);
 #if ENABLE_SPOOF_SOURCE
   if (success)
